@@ -4,6 +4,7 @@
 #include "efa_unit_tests.h"
 #include "rdm/efa_rdm_cq.h"
 #include "efa_rdm_pke_utils.h"
+#include "efa_data_path_direct_entry.h"
 
 /**
  * @brief Verify the EFA RDM endpoint correctly parses the host id string
@@ -105,12 +106,11 @@ void test_efa_rdm_ep_handshake_exchange_host_id(struct efa_resource **state, uin
 	struct efa_resource *resource = *state;
 	struct efa_unit_test_handshake_pkt_attr pkt_attr = {0};
 	struct fi_cq_data_entry cq_entry;
-	struct ibv_qp_ex *ibv_qp;
 	struct efa_rdm_ep *efa_rdm_ep;
 	struct efa_rdm_pke *pkt_entry;
 	uint64_t actual_peer_host_id = UINT64_MAX;
 	struct efa_rdm_cq *efa_rdm_cq;
-	struct ibv_cq_ex *ibv_cqx;
+	struct efa_ibv_cq *ibv_cq;
 
 	g_efa_unit_test_mocks.local_host_id = local_host_id;
 	g_efa_unit_test_mocks.peer_host_id = peer_host_id;
@@ -122,7 +122,7 @@ void test_efa_rdm_ep_handshake_exchange_host_id(struct efa_resource **state, uin
 
 	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
 	efa_rdm_cq = container_of(resource->cq, struct efa_rdm_cq, efa_cq.util_cq.cq_fid.fid);
-	ibv_cqx = efa_rdm_cq->efa_cq.ibv_cq.ibv_cq_ex;
+	ibv_cq = &(efa_rdm_cq->efa_cq.ibv_cq);
 
 	efa_rdm_ep->host_id = g_efa_unit_test_mocks.local_host_id;
 
@@ -157,51 +157,51 @@ void test_efa_rdm_ep_handshake_exchange_host_id(struct efa_resource **state, uin
 	pkt_attr.device_version = 0xefa0;
 	efa_unit_test_handshake_pkt_construct(pkt_entry, &pkt_attr);
 
-	ibv_qp = efa_rdm_ep->base_ep.qp->ibv_qp_ex;
-	ibv_qp->wr_start = &efa_mock_ibv_wr_start_no_op;
+	/* Setup QP mocks */
+	g_efa_unit_test_mocks.efa_qp_wr_start = &efa_mock_efa_qp_wr_start_no_op;
 	/* this mock will save the send work request (wr) in a global array */
-	ibv_qp->wr_send = &efa_mock_ibv_wr_send_verify_handshake_pkt_local_host_id_and_save_wr;
-	ibv_qp->wr_set_inline_data_list = &efa_mock_ibv_wr_set_inline_data_list_no_op;
-	ibv_qp->wr_set_sge_list = &efa_mock_ibv_wr_set_sge_list_no_op;
-	ibv_qp->wr_set_ud_addr = &efa_mock_ibv_wr_set_ud_addr_no_op;
-	ibv_qp->wr_complete = &efa_mock_ibv_wr_complete_no_op;
-	expect_function_call(efa_mock_ibv_wr_send_verify_handshake_pkt_local_host_id_and_save_wr);
+	g_efa_unit_test_mocks.efa_qp_wr_send = &efa_mock_efa_qp_wr_send_verify_handshake_pkt_local_host_id_and_save_wr;
+	g_efa_unit_test_mocks.efa_qp_wr_set_inline_data_list = &efa_mock_efa_qp_wr_set_inline_data_list_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_set_sge_list = &efa_mock_efa_qp_wr_set_sge_list_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_set_ud_addr = &efa_mock_efa_qp_wr_set_ud_addr_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_complete = &efa_mock_efa_qp_wr_complete_no_op;
+	expect_function_call(efa_mock_efa_qp_wr_send_verify_handshake_pkt_local_host_id_and_save_wr);
 
-	/* Setup CQ */
-	ibv_cqx->end_poll = &efa_mock_ibv_end_poll_check_mock;
-	ibv_cqx->next_poll = &efa_mock_ibv_next_poll_check_function_called_and_return_mock;
-	ibv_cqx->read_byte_len = &efa_mock_ibv_read_byte_len_return_mock;
-	ibv_cqx->read_opcode = &efa_mock_ibv_read_opcode_return_mock;
-	ibv_cqx->read_slid = &efa_mock_ibv_read_slid_return_mock;
-	ibv_cqx->read_src_qp = &efa_mock_ibv_read_src_qp_return_mock;
-	ibv_cqx->read_qp_num = &efa_mock_ibv_read_qp_num_return_mock;
-	ibv_cqx->read_wc_flags = &efa_mock_ibv_read_wc_flags_return_mock;
-	ibv_cqx->read_vendor_err = &efa_mock_ibv_read_vendor_err_return_mock;
-	ibv_cqx->start_poll = &efa_mock_ibv_start_poll_return_mock;
-	ibv_cqx->status = IBV_WC_SUCCESS;
-	ibv_cqx->wr_id = (uintptr_t)pkt_entry;
-	expect_function_call(efa_mock_ibv_next_poll_check_function_called_and_return_mock);
+	/* Setup CQ mocks */
+	g_efa_unit_test_mocks.efa_ibv_cq_end_poll = &efa_mock_efa_ibv_cq_end_poll_check_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_next_poll = &efa_mock_efa_ibv_cq_next_poll_check_function_called_and_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_byte_len = &efa_mock_efa_ibv_cq_wc_read_byte_len_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_opcode = &efa_mock_efa_ibv_cq_wc_read_opcode_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_slid = &efa_mock_efa_ibv_cq_wc_read_slid_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_src_qp = &efa_mock_efa_ibv_cq_wc_read_src_qp_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_qp_num = &efa_mock_efa_ibv_cq_wc_read_qp_num_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_wc_flags = &efa_mock_efa_ibv_cq_wc_read_wc_flags_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_wc_read_vendor_err = &efa_mock_efa_ibv_cq_wc_read_vendor_err_return_mock;
+	g_efa_unit_test_mocks.efa_ibv_cq_start_poll = &efa_mock_efa_ibv_cq_start_poll_return_mock;
+	ibv_cq->ibv_cq_ex->status = IBV_WC_SUCCESS;
+	ibv_cq->ibv_cq_ex->wr_id = (uintptr_t)pkt_entry;
+	expect_function_call(efa_mock_efa_ibv_cq_next_poll_check_function_called_and_return_mock);
 
 	/* Receive handshake packet */
-	will_return(efa_mock_ibv_end_poll_check_mock, NULL);
-	will_return(efa_mock_ibv_next_poll_check_function_called_and_return_mock, ENOENT);
-	will_return(efa_mock_ibv_read_byte_len_return_mock, pkt_entry->pkt_size);
-	will_return(efa_mock_ibv_read_opcode_return_mock, IBV_WC_RECV);
-	will_return(efa_mock_ibv_read_qp_num_return_mock, efa_rdm_ep->base_ep.qp->qp_num);
-	will_return(efa_mock_ibv_read_wc_flags_return_mock, 0);
-	will_return(efa_mock_ibv_read_slid_return_mock, efa_rdm_ep_get_peer_ahn(efa_rdm_ep, peer_addr));
-	will_return(efa_mock_ibv_read_src_qp_return_mock, raw_addr.qpn);
-	will_return(efa_mock_ibv_start_poll_return_mock, IBV_WC_SUCCESS);
+	will_return(efa_mock_efa_ibv_cq_end_poll_check_mock, NULL);
+	will_return(efa_mock_efa_ibv_cq_next_poll_check_function_called_and_return_mock, ENOENT);
+	will_return(efa_mock_efa_ibv_cq_wc_read_byte_len_return_mock, pkt_entry->pkt_size);
+	will_return(efa_mock_efa_ibv_cq_wc_read_opcode_return_mock, IBV_WC_RECV);
+	will_return(efa_mock_efa_ibv_cq_wc_read_qp_num_return_mock, efa_rdm_ep->base_ep.qp->qp_num);
+	will_return(efa_mock_efa_ibv_cq_wc_read_wc_flags_return_mock, 0);
+	will_return(efa_mock_efa_ibv_cq_wc_read_slid_return_mock, efa_rdm_ep_get_peer_ahn(efa_rdm_ep, peer_addr));
+	will_return(efa_mock_efa_ibv_cq_wc_read_src_qp_return_mock, raw_addr.qpn);
+	will_return(efa_mock_efa_ibv_cq_start_poll_return_mock, IBV_WC_SUCCESS);
 
 	/**
 	 * Fire away handshake packet.
 	 * Because we don't care if it fails(there is no receiver!), mark it as failed to make mocking simpler.
 	 */
-	will_return(efa_mock_ibv_end_poll_check_mock, NULL);
-	will_return(efa_mock_ibv_read_opcode_return_mock, IBV_WC_SEND);
-	will_return(efa_mock_ibv_read_qp_num_return_mock, efa_rdm_ep->base_ep.qp->qp_num);
-	will_return(efa_mock_ibv_read_vendor_err_return_mock, FI_EFA_ERR_OTHER);
-	will_return(efa_mock_ibv_start_poll_return_mock, IBV_WC_SUCCESS);
+	will_return(efa_mock_efa_ibv_cq_end_poll_check_mock, NULL);
+	will_return(efa_mock_efa_ibv_cq_wc_read_opcode_return_mock, IBV_WC_SEND);
+	will_return(efa_mock_efa_ibv_cq_wc_read_qp_num_return_mock, efa_rdm_ep->base_ep.qp->qp_num);
+	will_return(efa_mock_efa_ibv_cq_wc_read_vendor_err_return_mock, FI_EFA_ERR_OTHER);
+	will_return(efa_mock_efa_ibv_cq_start_poll_return_mock, IBV_WC_SUCCESS);
 
 	/* Progress the recv wr first to process the received handshake packet. */
 	cq_read_recv_ret = fi_cq_read(resource->cq, &cq_entry, 1);
@@ -212,8 +212,8 @@ void test_efa_rdm_ep_handshake_exchange_host_id(struct efa_resource **state, uin
 	 * We need to poll the CQ twice explicitly to point the CQE
 	 * to the saved send wr in handshake
 	 */
-	ibv_cqx->status = IBV_WC_GENERAL_ERR;
-	ibv_cqx->wr_id = (uintptr_t)g_ibv_submitted_wr_id_vec[0];
+	ibv_cq->ibv_cq_ex->status = IBV_WC_GENERAL_ERR;
+	ibv_cq->ibv_cq_ex->wr_id = (uintptr_t)g_ibv_submitted_wr_id_vec[0];
 
 	/* Progress the send wr to clean up outstanding tx ops */
 	cq_read_send_ret = fi_cq_read(resource->cq, &cq_entry, 1);
@@ -226,7 +226,12 @@ void test_efa_rdm_ep_handshake_exchange_host_id(struct efa_resource **state, uin
 	assert_true(actual_peer_host_id == g_efa_unit_test_mocks.peer_host_id);
 
 	/* Device version should be stored after handshake */
-        assert_int_equal(peer->device_version, 0xefa0);
+	assert_int_equal(peer->device_version, 0xefa0);
+
+	/* reset the mocked cq before it's polled by ep close */
+	will_return_always(efa_mock_efa_ibv_cq_start_poll_return_mock, ENOENT);
+	assert_int_equal(fi_close(&resource->ep->fid), 0);
+	resource->ep = NULL;
 }
 #else
 void test_efa_rdm_ep_handshake_exchange_host_id() {
@@ -1456,7 +1461,7 @@ void test_efa_rdm_ep_rx_refill_threshold_larger_than_rx_size(struct efa_resource
 }
 
 /**
- * @brief when unsolicited write recv is supported (by device + env),
+ * @brief when unsolicited write recv is supported (checked by cq),
  * efa_rdm_ep_support_unsolicited_write_recv
  * should return true, otherwise it should return false
  *
@@ -1466,13 +1471,15 @@ void test_efa_rdm_ep_rx_refill_threshold_larger_than_rx_size(struct efa_resource
 void test_efa_rdm_ep_support_unsolicited_write_recv(struct efa_resource **state)
 {
 	struct efa_rdm_ep *efa_rdm_ep;
+	struct efa_cq *efa_cq;
 	struct efa_resource *resource = *state;
 
 	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
 
 	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+	efa_cq = container_of(resource->cq, struct efa_cq, util_cq.cq_fid);
 
-	assert_int_equal(efa_use_unsolicited_write_recv(),
+	assert_int_equal(efa_cq->ibv_cq.unsolicited_write_recv_enabled,
 			 efa_rdm_ep_support_unsolicited_write_recv(efa_rdm_ep));
 }
 
@@ -1734,4 +1741,169 @@ void test_efa_ep_bind_and_enable(struct efa_resource **state)
 	assert_true(efa_ep->efa_qp_enabled);
 	/* we shouldn't have user recv qp for efa-direct */
 	assert_true(efa_ep->user_recv_qp == NULL);
-}
\ No newline at end of file
+}
+
+#if HAVE_EFA_DATA_PATH_DIRECT
+
+/**
+ * @brief qp's data_path_direct status should be consistent
+ * with cq's status
+ *
+ * This test is against efa-direct fabric
+ *
+ * @param state unit test resources
+ */
+static
+void test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_impl(struct efa_resource **state, bool data_path_direct_enabled)
+{
+	struct efa_resource *resource = *state;
+	struct efa_cq *efa_cq;
+	bool data_path_direct_enabled_orig;
+	struct fid_ep *ep;
+	struct efa_base_ep *efa_ep;
+
+	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_DIRECT_FABRIC_NAME);
+	efa_cq = container_of(resource->cq, struct efa_cq, util_cq.cq_fid);
+
+	/* recover the cq boolean */
+	data_path_direct_enabled_orig = efa_cq->ibv_cq.data_path_direct_enabled;
+	efa_cq->ibv_cq.data_path_direct_enabled = data_path_direct_enabled;
+
+	/* open a test ep */
+	assert_int_equal(fi_endpoint(resource->domain, resource->info, &ep, NULL), 0);
+	efa_ep = container_of(ep, struct efa_base_ep, util_ep.ep_fid);
+	assert_int_equal(fi_ep_bind(ep, &resource->cq->fid, FI_SEND | FI_RECV), 0);
+	assert_int_equal(fi_enable(ep), 0);
+
+	assert_true(efa_ep->qp->data_path_direct_enabled == data_path_direct_enabled);
+
+	assert_int_equal(fi_close(&ep->fid), 0);
+
+	/* recover the mocked boolean */
+	efa_cq->ibv_cq.data_path_direct_enabled = data_path_direct_enabled_orig;
+}
+
+void test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_happy(struct efa_resource **state)
+{
+	test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_impl(state, true);
+}
+
+void test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_unhappy(struct efa_resource **state)
+{
+	test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_impl(state, false);
+}
+#else
+
+/* No value to test this, already covered by test_efa_rdm_ep_data_path_direct_ops */
+void test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_happy(struct efa_resource **state)
+{
+	skip();
+}
+
+/* No value to test this, already covered by test_efa_rdm_ep_data_path_direct_ops */
+void test_efa_ep_data_path_direct_equal_to_cq_data_path_direct_unhappy(struct efa_resource **state)
+{
+	skip();
+}
+
+#endif /* HAVE_EFA_DIRECT_CQ */
+
+
+/**
+ * @brief Test qp's data_path_direct status for efa-rdm ep
+ * Currently, data_path_direct should always be disabled by efa-rdm.
+ *
+ * @param state pointer of efa_resource
+ */
+void test_efa_rdm_ep_data_path_direct_disabled(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_base_ep *efa_ep;
+
+	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
+
+	efa_ep = container_of(resource->ep, struct efa_base_ep, util_ep.ep_fid);
+
+	assert_false(efa_ep->qp->data_path_direct_enabled);
+}
+
+/**
+ * @brief Verify endpoint lock uses no-op locking with FI_THREAD_COMPLETION
+ *
+ * @param[in] state struct efa_resource managed by the framework
+ */
+void test_efa_ep_lock_type_no_op(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_base_ep *efa_base_ep;
+	resource->hints = efa_unit_test_alloc_hints(FI_EP_RDM, EFA_DIRECT_FABRIC_NAME);
+	assert_non_null(resource->hints);
+	resource->hints->domain_attr->threading = FI_THREAD_COMPLETION;
+	efa_unit_test_resource_construct_with_hints(resource, FI_EP_RDM, FI_VERSION(2, 0), resource->hints, false, true);
+
+	efa_base_ep = container_of(resource->ep, struct efa_base_ep, util_ep.ep_fid);
+	assert_int_equal(efa_base_ep->util_ep.lock.lock_type, OFI_LOCK_NOOP);
+}
+
+/**
+ * @brief Verify endpoint lock uses mutex locking with FI_THREAD_SAFE
+ *
+ * @param[in] state struct efa_resource managed by the framework
+ */
+void test_efa_ep_lock_type_mutex(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_base_ep *efa_base_ep;
+	resource->hints = efa_unit_test_alloc_hints(FI_EP_RDM, EFA_DIRECT_FABRIC_NAME);
+	assert_non_null(resource->hints);
+	resource->hints->domain_attr->threading = FI_THREAD_SAFE;
+	efa_unit_test_resource_construct_with_hints(resource, FI_EP_RDM, FI_VERSION(2, 0), resource->hints, false, true);
+
+	efa_base_ep = container_of(resource->ep, struct efa_base_ep, util_ep.ep_fid);
+	assert_int_equal(efa_base_ep->util_ep.lock.lock_type, OFI_LOCK_MUTEX);
+}
+
+/**
+ * @brief Test that EFA RDM endpoints created with same domain but different info
+ * have different shm_ep atrributes
+ *
+ * @param[in] state struct efa_resource managed by the framework
+ */
+void test_efa_rdm_ep_shm_ep_different_info(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_rdm_ep *efa_rdm_ep1, *efa_rdm_ep2;
+	struct fid_ep *ep2;
+	struct util_ep *shm_util_ep1, *shm_util_ep2;
+	struct fi_info *info2;
+	int ret;
+
+	resource->hints = efa_unit_test_alloc_hints(FI_EP_RDM, EFA_FABRIC_NAME);
+	efa_unit_test_resource_construct_with_hints(resource, FI_EP_RDM, FI_VERSION(1, 14), resource->hints, false, true);
+
+	/* Create second endpoint with different info but same domain */
+	info2 = fi_dupinfo(resource->info);
+	assert_non_null(info2);
+	info2->tx_attr->op_flags = FI_DELIVERY_COMPLETE;
+
+	ret = fi_endpoint(resource->domain, info2, &ep2, NULL);
+	assert_int_equal(ret, 0);
+
+	efa_rdm_ep1 = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+	efa_rdm_ep2 = container_of(ep2, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+
+	if (efa_env.enable_shm_transfer) {
+		assert_non_null(efa_rdm_ep1->shm_ep);
+		assert_non_null(efa_rdm_ep2->shm_ep);
+
+		shm_util_ep1 = container_of(efa_rdm_ep1->shm_ep, struct util_ep, ep_fid);
+		shm_util_ep2 = container_of(efa_rdm_ep2->shm_ep, struct util_ep, ep_fid);
+
+		assert_int_not_equal(shm_util_ep1->tx_op_flags, shm_util_ep2->tx_op_flags);
+		assert_true(shm_util_ep1->tx_op_flags & FI_COMPLETION);
+		assert_true(shm_util_ep2->tx_op_flags & FI_DELIVERY_COMPLETE);
+	}
+
+	fi_close(&ep2->fid);
+	fi_freeinfo(info2);
+}
