@@ -30,14 +30,17 @@ void test_efa_rdm_ope_prepare_to_post_send_impl(struct efa_resource *resource,
 
 	mock_mr.peer.iface = iface;
 
+	struct efa_rdm_ep *efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+	struct efa_rdm_peer *peer = efa_rdm_ep_get_peer(efa_rdm_ep, 0);
+
 	memset(&mock_txe, 0, sizeof(mock_txe));
 	mock_txe.total_len = total_len;
-	mock_txe.addr = addr;
+	mock_txe.peer = peer;
 	mock_txe.iov_count = 1;
 	mock_txe.iov[0].iov_base = NULL;
 	mock_txe.iov[0].iov_len = 9000;
 	mock_txe.desc[0] = &mock_mr;
-	mock_txe.ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+	mock_txe.ep = efa_rdm_ep;
 	mock_txe.peer = &mock_peer;
 
 	err = efa_rdm_ope_prepare_to_post_send(&mock_txe,
@@ -237,7 +240,6 @@ void test_efa_rdm_ope_prepare_to_post_send_cuda_memory_align128(struct efa_resou
 void test_efa_rdm_ope_post_write_0_byte(struct efa_resource **state)
 {
 	struct efa_resource *resource = *state;
-	struct ibv_qp_ex *ibv_qpx;
 	struct efa_unit_test_buff local_buff;
 	struct efa_ep_addr raw_addr;
 	struct efa_rdm_ope mock_txe;
@@ -254,10 +256,13 @@ void test_efa_rdm_ope_post_write_0_byte(struct efa_resource **state)
 	ret = fi_av_insert(resource->av, &raw_addr, 1, &addr, 0 /* flags */, NULL /* context */);
 	assert_int_equal(ret, 1);
 
+	struct efa_rdm_ep *efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+	struct efa_rdm_peer *peer = efa_rdm_ep_get_peer(efa_rdm_ep, 0);
+
 	efa_unit_test_buff_construct(&local_buff, resource, 4096 /* buff_size */);
 	memset(&mock_txe, 0, sizeof(mock_txe));
 	mock_txe.total_len = 0;
-	mock_txe.addr = addr;
+	mock_txe.peer = peer;
 	mock_txe.iov_count = 1;
 	mock_txe.iov[0].iov_base = local_buff.buff;
 	mock_txe.iov[0].iov_len = 0;
@@ -267,14 +272,13 @@ void test_efa_rdm_ope_post_write_0_byte(struct efa_resource **state)
 	mock_txe.rma_iov[0].key = 123456;
 	mock_txe.rma_iov[0].len = 0;
 
-	mock_txe.ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+	mock_txe.ep = efa_rdm_ep;
 
-	ibv_qpx = mock_txe.ep->base_ep.qp->ibv_qp_ex;
-	ibv_qpx->wr_start = &efa_mock_ibv_wr_start_no_op;
-	ibv_qpx->wr_rdma_write = &efa_mock_ibv_wr_rdma_write_save_wr;
-	ibv_qpx->wr_set_sge_list = &efa_mock_ibv_wr_set_sge_list_no_op;
-	ibv_qpx->wr_set_ud_addr = &efa_mock_ibv_wr_set_ud_addr_no_op;
-	ibv_qpx->wr_complete = &efa_mock_ibv_wr_complete_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_start = &efa_mock_efa_qp_wr_start_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_rdma_write = &efa_mock_efa_qp_wr_rdma_write_save_wr;
+	g_efa_unit_test_mocks.efa_qp_wr_set_sge_list = &efa_mock_efa_qp_wr_set_sge_list_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_set_ud_addr = &efa_mock_efa_qp_wr_set_ud_addr_no_op;
+	g_efa_unit_test_mocks.efa_qp_wr_complete = &efa_mock_efa_qp_wr_complete_no_op;
 
 	assert_int_equal(g_ibv_submitted_wr_id_cnt, 0);
 	err = efa_rdm_ope_post_remote_write(&mock_txe);
@@ -325,7 +329,7 @@ void test_efa_rdm_rxe_post_local_read_or_queue_impl(struct efa_resource *resourc
 	assert_non_null(pkt_entry);
 	pkt_entry->payload = pkt_entry->wiredata;
 
-	rxe = efa_rdm_ep_alloc_rxe(efa_rdm_ep, FI_ADDR_UNSPEC, ofi_op_tagged);
+	rxe = efa_rdm_ep_alloc_rxe(efa_rdm_ep, NULL, ofi_op_tagged);
 	cuda_mr.peer.iface = FI_HMEM_CUDA;
 
 	rxe->desc[0] = &cuda_mr;
@@ -482,6 +486,7 @@ void test_efa_rdm_rxe_map(struct efa_resource **state)
 {
 	struct efa_resource *resource = *state;
 	struct efa_rdm_ope *rxe;
+	struct efa_rdm_peer *peer;
 	struct efa_rdm_ep *efa_rdm_ep;
 
 	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
@@ -496,11 +501,14 @@ void test_efa_rdm_rxe_map(struct efa_resource **state)
 	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep,
 				  base_ep.util_ep.ep_fid);
 
-	efa_rdm_rxe_map_insert(&efa_rdm_ep->rxe_map, rxe->msg_id, rxe->addr,
-			       rxe);
-	assert_true(rxe->rxe_map == &efa_rdm_ep->rxe_map);
-	assert_true(rxe == efa_rdm_rxe_map_lookup(rxe->rxe_map, rxe->msg_id,
-						  rxe->addr));
+	/* efa_unit_test_alloc_rxe only inserts one address.
+	 * So fi_addr is guaranteed to be 0
+	 */
+	peer = efa_rdm_ep_get_peer(efa_rdm_ep, 0);
+
+	efa_rdm_rxe_map_insert(&peer->rxe_map, rxe->msg_id, rxe);
+	assert_true(rxe->rxe_map == &peer->rxe_map);
+	assert_true(rxe == efa_rdm_rxe_map_lookup(rxe->rxe_map, rxe->msg_id));
 
 	efa_rdm_rxe_release(rxe);
 
@@ -611,4 +619,157 @@ void test_efa_rdm_txe_prepare_local_read_pkt_entry(struct efa_resource **state)
 	 * assertion errors.
 	 */
 	assert_int_equal(fi_close(&ep->fid), 0);
-}
\ No newline at end of file
+}
+/**
+ * @brief Test that queue flags are properly cleaned up after error handling
+ *
+ * This test verifies that queue flags are properly cleaned up after
+ * dlist_remove in error handling functions.
+ */
+void test_efa_rdm_txe_handle_error_queue_flags_cleanup(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_rdm_ope *txe;
+	struct efa_rdm_ep *efa_rdm_ep;
+
+	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
+
+	txe = efa_unit_test_alloc_txe(resource, ofi_op_write);
+	assert_non_null(txe);
+
+	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+
+	/* Set up txe with queued flags */
+	txe->internal_flags |= EFA_RDM_OPE_QUEUED_CTRL;
+	dlist_insert_tail(&txe->queued_entry, &efa_rdm_ep_domain(efa_rdm_ep)->ope_queued_list);
+
+	/* Verify txe is in queued list */
+	assert_int_equal(efa_unit_test_get_dlist_length(&efa_rdm_ep_domain(efa_rdm_ep)->ope_queued_list), 1);
+	assert_true(txe->internal_flags & EFA_RDM_OPE_QUEUED_CTRL);
+
+	/* Handle error - this should clean up queue flags */
+	efa_rdm_txe_handle_error(txe, FI_ENOTCONN, EFA_IO_COMP_STATUS_LOCAL_ERROR_UNREACH_REMOTE);
+
+	/* Verify queue flags are cleaned up */
+	assert_false(txe->internal_flags & EFA_RDM_OPE_QUEUED_CTRL);
+	assert_true(dlist_empty(&efa_rdm_ep_domain(efa_rdm_ep)->ope_queued_list));
+
+	/* Release should not cause duplicate dlist_remove */
+	efa_rdm_txe_release(txe);
+}
+
+/**
+ * @brief Test that queue flags are properly cleaned up for rxe error handling
+ */
+void test_efa_rdm_rxe_handle_error_queue_flags_cleanup(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_rdm_ope *rxe;
+	struct efa_rdm_ep *efa_rdm_ep;
+
+	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
+
+	rxe = efa_unit_test_alloc_rxe(resource, ofi_op_tagged);
+	assert_non_null(rxe);
+
+	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+
+	/* Set up rxe with queued flags */
+	rxe->internal_flags |= EFA_RDM_OPE_QUEUED_READ;
+	dlist_insert_tail(&rxe->queued_entry, &efa_rdm_ep_domain(efa_rdm_ep)->ope_queued_list);
+
+	/* Verify rxe is in queued list */
+	assert_int_equal(efa_unit_test_get_dlist_length(&efa_rdm_ep_domain(efa_rdm_ep)->ope_queued_list), 1);
+	assert_true(rxe->internal_flags & EFA_RDM_OPE_QUEUED_READ);
+
+	/* Handle error - this should clean up queue flags */
+	efa_rdm_rxe_handle_error(rxe, FI_ENOTCONN, EFA_IO_COMP_STATUS_LOCAL_ERROR_UNREACH_REMOTE);
+
+	/* Verify queue flags are cleaned up */
+	assert_false(rxe->internal_flags & EFA_RDM_OPE_QUEUED_READ);
+	assert_true(dlist_empty(&efa_rdm_ep_domain(efa_rdm_ep)->ope_queued_list));
+
+	/* Release should not cause duplicate dlist_remove */
+	efa_rdm_rxe_release(rxe);
+}
+
+/**
+ * @brief Test error state prevents duplicate error handling for txe
+ *
+ * This test verifies that a new error state prevents duplicate error handling.
+ */
+void test_efa_rdm_txe_handle_error_duplicate_prevention(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_rdm_ope *txe;
+	struct efa_rdm_ep *efa_rdm_ep;
+
+	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
+
+	txe = efa_unit_test_alloc_txe(resource, ofi_op_write);
+	assert_non_null(txe);
+
+	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+
+	/* Set txe to EFA_RDM_OPE_SEND state and add to longcts_send_list */
+	txe->state = EFA_RDM_OPE_SEND;
+	dlist_insert_tail(&txe->entry, &efa_rdm_ep_domain(efa_rdm_ep)->ope_longcts_send_list);
+
+	/* Verify txe is in the list */
+	assert_int_equal(efa_unit_test_get_dlist_length(&efa_rdm_ep_domain(efa_rdm_ep)->ope_longcts_send_list), 1);
+
+	/* First error handling call */
+	efa_rdm_txe_handle_error(txe, FI_ENOTCONN, EFA_IO_COMP_STATUS_LOCAL_ERROR_UNREACH_REMOTE);
+
+	/* Verify txe is removed from list and in error state */
+	assert_true(dlist_empty(&efa_rdm_ep_domain(efa_rdm_ep)->ope_longcts_send_list));
+	assert_int_equal(txe->state, EFA_RDM_OPE_ERR);
+
+	/* Second error handling call should be a no-op */
+	efa_rdm_txe_handle_error(txe, FI_EAGAIN, EFA_IO_COMP_STATUS_LOCAL_ERROR_BAD_LENGTH);
+
+	/* State should remain EFA_RDM_OPE_ERR */
+	assert_int_equal(txe->state, EFA_RDM_OPE_ERR);
+
+	efa_rdm_txe_release(txe);
+}
+
+/**
+ * @brief Test error state prevents duplicate error handling for rxe
+ */
+void test_efa_rdm_rxe_handle_error_duplicate_prevention(struct efa_resource **state)
+{
+	struct efa_resource *resource = *state;
+	struct efa_rdm_ope *rxe;
+	struct efa_rdm_ep *efa_rdm_ep;
+
+	efa_unit_test_resource_construct(resource, FI_EP_RDM, EFA_FABRIC_NAME);
+
+	rxe = efa_unit_test_alloc_rxe(resource, ofi_op_tagged);
+	assert_non_null(rxe);
+
+	efa_rdm_ep = container_of(resource->ep, struct efa_rdm_ep, base_ep.util_ep.ep_fid);
+
+	/* Set rxe to EFA_RDM_OPE_SEND state and add to longcts_send_list */
+	rxe->state = EFA_RDM_OPE_SEND;
+	dlist_insert_tail(&rxe->entry, &efa_rdm_ep_domain(efa_rdm_ep)->ope_longcts_send_list);
+
+	/* Verify rxe is in the list */
+	assert_int_equal(efa_unit_test_get_dlist_length(&efa_rdm_ep_domain(efa_rdm_ep)->ope_longcts_send_list), 1);
+
+	/* First error handling call */
+	efa_rdm_rxe_handle_error(rxe, FI_ENOTCONN, EFA_IO_COMP_STATUS_LOCAL_ERROR_UNREACH_REMOTE);
+
+	/* Verify rxe is removed from list and in error state */
+	assert_true(dlist_empty(&efa_rdm_ep_domain(efa_rdm_ep)->ope_longcts_send_list));
+	assert_int_equal(rxe->state, EFA_RDM_OPE_ERR);
+
+	/* Second error handling call should be a no-op */
+	efa_rdm_rxe_handle_error(rxe, FI_EAGAIN, EFA_IO_COMP_STATUS_LOCAL_ERROR_BAD_LENGTH);
+
+	/* State should remain EFA_RDM_OPE_ERR */
+	assert_int_equal(rxe->state, EFA_RDM_OPE_ERR);
+
+	efa_rdm_rxe_release(rxe);
+}
+
