@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2013-2014 Intel Corporation. All rights reserved.
  * Copyright (c) 2016 Cisco Systems, Inc. All rights reserved.
+ * Copyright (c) 2017 DataDirect Networks, Inc. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -46,7 +47,7 @@
 #include <ifaddrs.h>
 #endif
 
-#include "fi_util.h"
+#include "ofi_util.h"
 #include "sock.h"
 #include "sock_util.h"
 
@@ -63,31 +64,6 @@ extern struct fi_ops_ep sock_ep_ops;
 extern struct fi_ops sock_ep_fi_ops;
 extern struct fi_ops_ep sock_ctx_ep_ops;
 
-extern const struct fi_domain_attr sock_domain_attr;
-extern const struct fi_fabric_attr sock_fabric_attr;
-
-const struct fi_tx_attr sock_stx_attr = {
-	.caps = SOCK_EP_RDM_CAP,
-	.mode = SOCK_MODE,
-	.op_flags = FI_TRANSMIT_COMPLETE,
-	.msg_order = SOCK_EP_MSG_ORDER,
-	.inject_size = SOCK_EP_MAX_INJECT_SZ,
-	.size = SOCK_EP_TX_SZ,
-	.iov_limit = SOCK_EP_MAX_IOV_LIMIT,
-	.rma_iov_limit = SOCK_EP_MAX_IOV_LIMIT,
-};
-
-const struct fi_rx_attr sock_srx_attr = {
-	.caps = SOCK_EP_RDM_CAP,
-	.mode = SOCK_MODE,
-	.op_flags = 0,
-	.msg_order = SOCK_EP_MSG_ORDER,
-	.comp_order = SOCK_EP_COMP_ORDER,
-	.total_buffered_recv = 0,
-	.size = SOCK_EP_MAX_MSG_SZ,
-	.iov_limit = SOCK_EP_MAX_IOV_LIMIT,
-};
-
 static void sock_tx_ctx_close(struct sock_tx_ctx *tx_ctx)
 {
 	if (tx_ctx->comp.send_cq)
@@ -127,8 +103,8 @@ static int sock_ctx_close(struct fid *fid)
 	case FI_CLASS_TX_CTX:
 		tx_ctx = container_of(fid, struct sock_tx_ctx, fid.ctx.fid);
 		sock_pe_remove_tx_ctx(tx_ctx);
-		atomic_dec(&tx_ctx->ep_attr->num_tx_ctx);
-		atomic_dec(&tx_ctx->domain->ref);
+		ofi_atomic_dec32(&tx_ctx->ep_attr->num_tx_ctx);
+		ofi_atomic_dec32(&tx_ctx->domain->ref);
 		sock_tx_ctx_close(tx_ctx);
 		sock_tx_ctx_free(tx_ctx);
 		break;
@@ -136,22 +112,22 @@ static int sock_ctx_close(struct fid *fid)
 	case FI_CLASS_RX_CTX:
 		rx_ctx = container_of(fid, struct sock_rx_ctx, ctx.fid);
 		sock_pe_remove_rx_ctx(rx_ctx);
-		atomic_dec(&rx_ctx->ep_attr->num_rx_ctx);
-		atomic_dec(&rx_ctx->domain->ref);
+		ofi_atomic_dec32(&rx_ctx->ep_attr->num_rx_ctx);
+		ofi_atomic_dec32(&rx_ctx->domain->ref);
 		sock_rx_ctx_close(rx_ctx);
 		sock_rx_ctx_free(rx_ctx);
 		break;
 
 	case FI_CLASS_STX_CTX:
 		tx_ctx = container_of(fid, struct sock_tx_ctx, fid.stx.fid);
-		atomic_dec(&tx_ctx->domain->ref);
+		ofi_atomic_dec32(&tx_ctx->domain->ref);
 		sock_pe_remove_tx_ctx(tx_ctx);
 		sock_tx_ctx_free(tx_ctx);
 		break;
 
 	case FI_CLASS_SRX_CTX:
 		rx_ctx = container_of(fid, struct sock_rx_ctx, ctx.fid);
-		atomic_dec(&rx_ctx->domain->ref);
+		ofi_atomic_dec32(&rx_ctx->domain->ref);
 		sock_pe_remove_rx_ctx(rx_ctx);
 		sock_rx_ctx_free(rx_ctx);
 		break;
@@ -219,29 +195,38 @@ static int sock_ctx_bind_cntr(struct fid *fid, struct fid *bfid, uint64_t flags)
 	switch (fid->fclass) {
 	case FI_CLASS_TX_CTX:
 		tx_ctx = container_of(fid, struct sock_tx_ctx, fid.ctx.fid);
-		if (flags & FI_SEND)
+		if (flags & FI_SEND) {
 			tx_ctx->comp.send_cntr = cntr;
+			sock_cntr_add_tx_ctx(cntr, tx_ctx);
+		}
 
-		if (flags & FI_READ)
+		if (flags & FI_READ) {
 			tx_ctx->comp.read_cntr = cntr;
+			sock_cntr_add_tx_ctx(cntr, tx_ctx);
+		}
 
-		if (flags & FI_WRITE)
+		if (flags & FI_WRITE) {
 			tx_ctx->comp.write_cntr = cntr;
-
-		sock_cntr_add_tx_ctx(cntr, tx_ctx);
+			sock_cntr_add_tx_ctx(cntr, tx_ctx);
+		}
 		break;
 
 	case FI_CLASS_RX_CTX:
 		rx_ctx = container_of(fid, struct sock_rx_ctx, ctx.fid);
-		if (flags & FI_RECV)
+		if (flags & FI_RECV) {
 			rx_ctx->comp.recv_cntr = cntr;
+			sock_cntr_add_rx_ctx(cntr, rx_ctx);
+		}
 
-		if (flags & FI_REMOTE_READ)
+		if (flags & FI_REMOTE_READ) {
 			rx_ctx->comp.rem_read_cntr = cntr;
+			sock_cntr_add_rx_ctx(cntr, rx_ctx);
+		}
 
-		if (flags & FI_REMOTE_WRITE)
+		if (flags & FI_REMOTE_WRITE) {
 			rx_ctx->comp.rem_write_cntr = cntr;
-		sock_cntr_add_rx_ctx(cntr, rx_ctx);
+			sock_cntr_add_rx_ctx(cntr, rx_ctx);
+		}
 		break;
 
 	default:
@@ -278,24 +263,24 @@ static int sock_ctx_enable(struct fid_ep *ep)
 	switch (ep->fid.fclass) {
 	case FI_CLASS_RX_CTX:
 		rx_ctx = container_of(ep, struct sock_rx_ctx, ctx.fid);
-		rx_ctx->enabled = 1;
 		sock_pe_add_rx_ctx(rx_ctx->domain->pe, rx_ctx);
 
-		if (!rx_ctx->ep_attr->listener.listener_thread &&
+		if (!rx_ctx->ep_attr->conn_handle.do_listen &&
 		    sock_conn_listen(rx_ctx->ep_attr)) {
 			SOCK_LOG_ERROR("failed to create listener\n");
 		}
+		rx_ctx->enabled = 1;
 		return 0;
 
 	case FI_CLASS_TX_CTX:
 		tx_ctx = container_of(ep, struct sock_tx_ctx, fid.ctx.fid);
-		tx_ctx->enabled = 1;
 		sock_pe_add_tx_ctx(tx_ctx->domain->pe, tx_ctx);
 
-		if (!tx_ctx->ep_attr->listener.listener_thread &&
+		if (!tx_ctx->ep_attr->conn_handle.do_listen &&
 		    sock_conn_listen(tx_ctx->ep_attr)) {
 			SOCK_LOG_ERROR("failed to create listener\n");
 		}
+		tx_ctx->enabled = 1;
 		return 0;
 
 	default:
@@ -467,7 +452,7 @@ static ssize_t sock_rx_ctx_cancel(struct sock_rx_ctx *rx_ctx, void *context)
 	struct sock_rx_entry *rx_entry;
 	struct sock_pe_entry pe_entry;
 
-	fastlock_acquire(&rx_ctx->lock);
+	ofi_mutex_lock(&rx_ctx->lock);
 	for (entry = rx_ctx->rx_entry_list.next;
 	     entry != &rx_ctx->rx_entry_list; entry = entry->next) {
 
@@ -486,14 +471,14 @@ static ssize_t sock_rx_ctx_cancel(struct sock_rx_ctx *rx_ctx, void *context)
 					pe_entry.flags |= FI_TAGGED;
 
 				if (sock_cq_report_error(pe_entry.comp->recv_cq,
-							  &pe_entry, 0, FI_ECANCELED,
-							  -FI_ECANCELED, NULL)) {
+							 &pe_entry, 0, FI_ECANCELED,
+							 -FI_ECANCELED, NULL, 0)) {
 					SOCK_LOG_ERROR("failed to report error\n");
 				}
 			}
 
 			if (rx_ctx->comp.recv_cntr)
-				sock_cntr_err_inc(rx_ctx->comp.recv_cntr);
+				fi_cntr_adderr(&rx_ctx->comp.recv_cntr->cntr_fid, 1);
 
 			dlist_remove(&rx_entry->entry);
 			sock_rx_release_entry(rx_entry);
@@ -501,7 +486,7 @@ static ssize_t sock_rx_ctx_cancel(struct sock_rx_ctx *rx_ctx, void *context)
 			break;
 		}
 	}
-	fastlock_release(&rx_ctx->lock);
+	ofi_mutex_unlock(&rx_ctx->lock);
 	return ret;
 }
 
@@ -587,9 +572,9 @@ static ssize_t sock_tx_size_left(struct fid_ep *ep)
 	if (!tx_ctx->enabled)
 		return -FI_EOPBADSTATE;
 
-	fastlock_acquire(&tx_ctx->wlock);
-	num_left = rbavail(&tx_ctx->rb)/SOCK_EP_TX_ENTRY_SZ;
-	fastlock_release(&tx_ctx->wlock);
+	ofi_mutex_lock(&tx_ctx->rb_lock);
+	num_left = ofi_rbavail(&tx_ctx->rb)/SOCK_EP_TX_ENTRY_SZ;
+	ofi_mutex_unlock(&tx_ctx->rb_lock);
 	return num_left;
 }
 
@@ -604,10 +589,38 @@ struct fi_ops_ep sock_ctx_ep_ops = {
 	.tx_size_left = sock_tx_size_left,
 };
 
+static int sock_eq_fid_match(struct dlist_entry *entry, const void *arg)
+{
+	struct sock_eq_entry *sock_eq_entry;
+	struct fi_eq_entry *eq_entry;
+	fid_t fid = (fid_t)arg;
+
+	sock_eq_entry = container_of(entry, struct sock_eq_entry, entry);
+	/* fi_eq_entry, fi_eq_cm_entry and fi_eq_err_entry all
+	 * have fid_t as first member */
+	eq_entry = (struct fi_eq_entry *)sock_eq_entry->event;
+	return (fid == eq_entry->fid);
+}
+
+static void sock_ep_clear_eq_list(struct dlistfd_head *list,
+				  struct fid_ep *ep_fid)
+{
+	struct dlist_entry *entry;
+
+	while (!dlistfd_empty(list)) {
+		entry = dlist_remove_first_match(&list->list, sock_eq_fid_match,
+						 ep_fid);
+		if (!entry)
+			break;
+		dlistfd_reset(list);
+		free(container_of(entry, struct sock_eq_entry, entry));
+	}
+}
+
 static int sock_ep_close(struct fid *fid)
 {
+	struct sock_conn_req_handle *handle;
 	struct sock_ep *sock_ep;
-	char c = 0;
 
 	switch (fid->fclass) {
 	case FI_CLASS_EP:
@@ -623,63 +636,70 @@ static int sock_ep_close(struct fid *fid)
 	}
 
 	if (sock_ep->is_alias) {
-		atomic_dec(&sock_ep->attr->ref);
+		ofi_atomic_dec32(&sock_ep->attr->ref);
 		return 0;
 	}
-	if (atomic_get(&sock_ep->attr->ref) || atomic_get(&sock_ep->attr->num_rx_ctx) ||
-	    atomic_get(&sock_ep->attr->num_tx_ctx))
+	if (ofi_atomic_get32(&sock_ep->attr->ref) ||
+	    ofi_atomic_get32(&sock_ep->attr->num_rx_ctx) ||
+	    ofi_atomic_get32(&sock_ep->attr->num_tx_ctx))
 		return -FI_EBUSY;
 
 	if (sock_ep->attr->ep_type == FI_EP_MSG) {
-		sock_ep->attr->cm.do_listen = 0;
-		if (ofi_write_socket(sock_ep->attr->cm.signal_fds[0], &c, 1) != 1)
-			SOCK_LOG_DBG("Failed to signal\n");
-
-		if (sock_ep->attr->cm.listener_thread &&
-		    pthread_join(sock_ep->attr->cm.listener_thread, NULL)) {
-			SOCK_LOG_ERROR("pthread join failed (%d)\n", errno);
+		if (sock_ep->attr->info.handle) {
+			handle = container_of(sock_ep->attr->info.handle,
+		                          struct sock_conn_req_handle, handle);
+			sock_ep_cm_wait_handle_finalized(&sock_ep->attr->domain->cm_head,
+			                                 handle);
+			free(handle->req);
+			free(handle);
 		}
-		ofi_close_socket(sock_ep->attr->cm.signal_fds[0]);
-		ofi_close_socket(sock_ep->attr->cm.signal_fds[1]);
 	} else {
 		if (sock_ep->attr->av)
-			atomic_dec(&sock_ep->attr->av->ref);
+			ofi_atomic_dec32(&sock_ep->attr->av->ref);
 	}
 	if (sock_ep->attr->av) {
-		fastlock_acquire(&sock_ep->attr->av->list_lock);
-		fid_list_remove(&sock_ep->attr->av->ep_list, &sock_ep->attr->lock, &sock_ep->ep.fid);
-		fastlock_release(&sock_ep->attr->av->list_lock);
+		ofi_mutex_lock(&sock_ep->attr->av->list_lock);
+		fid_list_remove(&sock_ep->attr->av->ep_list,
+				&sock_ep->attr->lock, &sock_ep->ep.fid);
+		ofi_mutex_unlock(&sock_ep->attr->av->list_lock);
 	}
 
 	pthread_mutex_lock(&sock_ep->attr->domain->pe->list_lock);
 	if (sock_ep->attr->tx_shared) {
-		fastlock_acquire(&sock_ep->attr->tx_ctx->lock);
+		ofi_mutex_lock(&sock_ep->attr->tx_ctx->lock);
 		dlist_remove(&sock_ep->attr->tx_ctx_entry);
-		fastlock_release(&sock_ep->attr->tx_ctx->lock);
+		ofi_mutex_unlock(&sock_ep->attr->tx_ctx->lock);
 	}
 
 	if (sock_ep->attr->rx_shared) {
-		fastlock_acquire(&sock_ep->attr->rx_ctx->lock);
+		ofi_mutex_lock(&sock_ep->attr->rx_ctx->lock);
 		dlist_remove(&sock_ep->attr->rx_ctx_entry);
-		fastlock_release(&sock_ep->attr->rx_ctx->lock);
+		ofi_mutex_unlock(&sock_ep->attr->rx_ctx->lock);
 	}
 	pthread_mutex_unlock(&sock_ep->attr->domain->pe->list_lock);
 
-	if (sock_ep->attr->listener.do_listen) {
-		sock_ep->attr->listener.do_listen = 0;
-		if (ofi_write_socket(sock_ep->attr->listener.signal_fds[0], &c, 1) != 1)
-			SOCK_LOG_DBG("Failed to signal\n");
-
-		if (sock_ep->attr->listener.listener_thread &&
-		     pthread_join(sock_ep->attr->listener.listener_thread, NULL)) {
-			SOCK_LOG_ERROR("pthread join failed (%d)\n", errno);
-		}
-
-		ofi_close_socket(sock_ep->attr->listener.signal_fds[0]);
-		ofi_close_socket(sock_ep->attr->listener.signal_fds[1]);
+	if (sock_ep->attr->conn_handle.do_listen) {
+		ofi_mutex_lock(&sock_ep->attr->domain->conn_listener.signal_lock);
+		ofi_epoll_del(sock_ep->attr->domain->conn_listener.epollfd,
+		             sock_ep->attr->conn_handle.sock);
+		sock_ep->attr->domain->conn_listener.removed_from_epollfd = true;
+		ofi_mutex_unlock(&sock_ep->attr->domain->conn_listener.signal_lock);
+		ofi_close_socket(sock_ep->attr->conn_handle.sock);
+		sock_ep->attr->conn_handle.do_listen = 0;
 	}
 
-	fastlock_destroy(&sock_ep->attr->cm.lock);
+	ofi_mutex_destroy(&sock_ep->attr->cm.lock);
+
+	if (sock_ep->attr->eq) {
+		ofi_mutex_lock(&sock_ep->attr->eq->lock);
+		sock_ep_clear_eq_list(&sock_ep->attr->eq->list,
+				      &sock_ep->ep);
+		/* Any err_data if present would be freed by
+		 * sock_eq_clean_err_data_list when EQ is closed */
+		sock_ep_clear_eq_list(&sock_ep->attr->eq->err_list,
+				      &sock_ep->ep);
+		ofi_mutex_unlock(&sock_ep->attr->eq->lock);
+	}
 
 	if (sock_ep->attr->fclass != FI_CLASS_SEP) {
 		if (!sock_ep->attr->tx_shared)
@@ -705,14 +725,13 @@ static int sock_ep_close(struct fid *fid)
 	if (sock_ep->attr->dest_addr)
 		free(sock_ep->attr->dest_addr);
 
-	fastlock_acquire(&sock_ep->attr->domain->pe->lock);
-	idm_reset(&sock_ep->attr->conn_idm);
-	idm_reset(&sock_ep->attr->av_idm);
+	ofi_mutex_lock(&sock_ep->attr->domain->pe->lock);
+	ofi_idm_reset(&sock_ep->attr->av_idm, NULL);
 	sock_conn_map_destroy(sock_ep->attr);
-	fastlock_release(&sock_ep->attr->domain->pe->lock);
+	ofi_mutex_unlock(&sock_ep->attr->domain->pe->lock);
 
-	atomic_dec(&sock_ep->attr->domain->ref);
-	fastlock_destroy(&sock_ep->attr->lock);
+	ofi_atomic_dec32(&sock_ep->attr->domain->ref);
+	ofi_mutex_destroy(&sock_ep->attr->lock);
 	free(sock_ep->attr);
 	free(sock_ep);
 	return 0;
@@ -720,7 +739,8 @@ static int sock_ep_close(struct fid *fid)
 
 static int sock_ep_bind(struct fid *fid, struct fid *bfid, uint64_t flags)
 {
-	int ret, i;
+	int ret;
+	size_t i;
 	struct sock_ep *ep;
 	struct sock_eq *eq;
 	struct sock_cq *cq;
@@ -826,7 +846,7 @@ static int sock_ep_bind(struct fid *fid, struct fid *bfid, uint64_t flags)
 			return -FI_EINVAL;
 
 		ep->attr->av = av;
-		atomic_inc(&av->ref);
+		ofi_atomic_inc32(&av->ref);
 
 		if (ep->attr->tx_ctx &&
 		    ep->attr->tx_ctx->fid.ctx.fid.fclass == FI_CLASS_TX_CTX) {
@@ -846,21 +866,21 @@ static int sock_ep_bind(struct fid *fid, struct fid *bfid, uint64_t flags)
 			if (ep->attr->rx_array[i])
 				ep->attr->rx_array[i]->av = av;
 		}
-		fastlock_acquire(&av->list_lock);
+		ofi_mutex_lock(&av->list_lock);
 		ret = fid_list_insert(&av->ep_list, &ep->attr->lock, &ep->ep.fid);
 		if (ret) {
 			SOCK_LOG_ERROR("Error in adding fid in the EP list\n");
-			fastlock_release(&av->list_lock);
+			ofi_mutex_unlock(&av->list_lock);
 			return ret;
 		}
-		fastlock_release(&av->list_lock);
+		ofi_mutex_unlock(&av->list_lock);
 		break;
 
 	case FI_CLASS_STX_CTX:
 		tx_ctx = container_of(bfid, struct sock_tx_ctx, fid.stx.fid);
-		fastlock_acquire(&tx_ctx->lock);
+		ofi_mutex_lock(&tx_ctx->lock);
 		dlist_insert_tail(&ep->attr->tx_ctx_entry, &tx_ctx->ep_list);
-		fastlock_release(&tx_ctx->lock);
+		ofi_mutex_unlock(&tx_ctx->lock);
 
 		ep->attr->tx_ctx->use_shared = 1;
 		ep->attr->tx_ctx->stx_ctx = tx_ctx;
@@ -868,9 +888,9 @@ static int sock_ep_bind(struct fid *fid, struct fid *bfid, uint64_t flags)
 
 	case FI_CLASS_SRX_CTX:
 		rx_ctx = container_of(bfid, struct sock_rx_ctx, ctx);
-		fastlock_acquire(&rx_ctx->lock);
+		ofi_mutex_lock(&rx_ctx->lock);
 		dlist_insert_tail(&ep->attr->rx_ctx_entry, &rx_ctx->ep_list);
-		fastlock_release(&rx_ctx->lock);
+		ofi_mutex_unlock(&rx_ctx->lock);
 
 		ep->attr->rx_ctx->use_shared = 1;
 		ep->attr->rx_ctx->srx_ctx = rx_ctx;
@@ -922,7 +942,7 @@ static int sock_ep_control(struct fid *fid, int command, void *arg)
 		new_ep->is_alias = 1;
 		memcpy(&new_ep->ep, &sock_ep->ep, sizeof(struct fid_ep));
 		*alias->fid = &new_ep->ep.fid;
-		atomic_inc(&new_ep->attr->ref);
+		ofi_atomic_inc32(&new_ep->attr->ref);
 		break;
 	case FI_GETOPSFLAG:
 		ret = sock_getopflags(&sock_ep->tx_attr, &sock_ep->rx_attr, (uint64_t *) arg);
@@ -956,7 +976,7 @@ struct fi_ops sock_ep_fi_ops = {
 
 int sock_ep_enable(struct fid_ep *ep)
 {
-	int i;
+	size_t i;
 	struct sock_ep *sock_ep;
 	struct sock_tx_ctx *tx_ctx;
 	struct sock_rx_ctx *rx_ctx;
@@ -992,16 +1012,20 @@ int sock_ep_enable(struct fid_ep *ep)
 		}
 	}
 
-	if (sock_ep->attr->ep_type != FI_EP_MSG &&
-	    !sock_ep->attr->listener.listener_thread && sock_conn_listen(sock_ep->attr))
-		SOCK_LOG_ERROR("cannot start connection thread\n");
+	if (sock_ep->attr->ep_type != FI_EP_MSG && !sock_ep->attr->conn_handle.do_listen) {
+		int ret = sock_conn_listen(sock_ep->attr);
+		if (ret) {
+			SOCK_LOG_ERROR("cannot start connection thread\n");
+			return ret;
+		}
+	}
 	sock_ep->attr->is_enabled = 1;
 	return 0;
 }
 
 int sock_ep_disable(struct fid_ep *ep)
 {
-	int i;
+	size_t i;
 	struct sock_ep *sock_ep;
 
 	sock_ep = container_of(ep, struct sock_ep, ep);
@@ -1062,7 +1086,7 @@ static int sock_ep_getopt(fid_t fid, int level, int optname,
 static int sock_ep_setopt(fid_t fid, int level, int optname,
 		       const void *optval, size_t optlen)
 {
-	int i;
+	size_t i;
 	struct sock_ep *sock_ep;
 	sock_ep = container_of(fid, struct sock_ep, ep.fid);
 
@@ -1095,12 +1119,17 @@ static int sock_ep_tx_ctx(struct fid_ep *ep, int index, struct fi_tx_attr *attr,
 
 	sock_ep = container_of(ep, struct sock_ep, ep);
 	if (sock_ep->attr->fclass != FI_CLASS_SEP ||
-		index >= sock_ep->attr->ep_attr.tx_ctx_cnt)
+		index >= (int)sock_ep->attr->ep_attr.tx_ctx_cnt)
 		return -FI_EINVAL;
 
 	if (attr) {
-		if (fi_check_tx_attr(&sock_prov, &sock_ep->tx_attr, attr))
+		if (ofi_check_tx_attr(&sock_prov, sock_ep->attr->info.tx_attr,
+				      attr, 0) ||
+		    ofi_check_attr_subset(&sock_prov,
+					  sock_ep->attr->info.tx_attr->caps,
+					  attr->caps & ~OFI_IGNORED_TX_CAPS)) {
 			return -FI_ENODATA;
+		}
 		tx_ctx = sock_tx_ctx_alloc(attr, context, 0);
 	} else {
 		tx_ctx = sock_tx_ctx_alloc(&sock_ep->tx_attr, context, 0);
@@ -1108,9 +1137,11 @@ static int sock_ep_tx_ctx(struct fid_ep *ep, int index, struct fi_tx_attr *attr,
 	if (!tx_ctx)
 		return -FI_ENOMEM;
 
-	tx_ctx->tx_id = index;
+	tx_ctx->tx_id = (uint16_t) index;
 	tx_ctx->ep_attr = sock_ep->attr;
 	tx_ctx->domain = sock_ep->attr->domain;
+	if (tx_ctx->rx_ctrl_ctx && tx_ctx->rx_ctrl_ctx->is_ctrl_ctx)
+		tx_ctx->rx_ctrl_ctx->domain = sock_ep->attr->domain;
 	tx_ctx->av = sock_ep->attr->av;
 	dlist_insert_tail(&sock_ep->attr->tx_ctx_entry, &tx_ctx->ep_list);
 
@@ -1123,8 +1154,8 @@ static int sock_ep_tx_ctx(struct fid_ep *ep, int index, struct fi_tx_attr *attr,
 
 	*tx_ep = &tx_ctx->fid.ctx;
 	sock_ep->attr->tx_array[index] = tx_ctx;
-	atomic_inc(&sock_ep->attr->num_tx_ctx);
-	atomic_inc(&sock_ep->attr->domain->ref);
+	ofi_atomic_inc32(&sock_ep->attr->num_tx_ctx);
+	ofi_atomic_inc32(&sock_ep->attr->domain->ref);
 	return 0;
 }
 
@@ -1136,12 +1167,15 @@ static int sock_ep_rx_ctx(struct fid_ep *ep, int index, struct fi_rx_attr *attr,
 
 	sock_ep = container_of(ep, struct sock_ep, ep);
 	if (sock_ep->attr->fclass != FI_CLASS_SEP ||
-		index >= sock_ep->attr->ep_attr.rx_ctx_cnt)
+		index >= (int)sock_ep->attr->ep_attr.rx_ctx_cnt)
 		return -FI_EINVAL;
 
 	if (attr) {
-		if (fi_check_rx_attr(&sock_prov, &sock_ep->rx_attr, attr))
+		if (ofi_check_rx_attr(&sock_prov, &sock_ep->attr->info, attr, 0) ||
+		    ofi_check_attr_subset(&sock_prov, sock_ep->attr->info.rx_attr->caps,
+					  attr->caps & ~OFI_IGNORED_RX_CAPS)) {
 			return -FI_ENODATA;
+		}
 		rx_ctx = sock_rx_ctx_alloc(attr, context, 0);
 	} else {
 		rx_ctx = sock_rx_ctx_alloc(&sock_ep->rx_attr, context, 0);
@@ -1149,7 +1183,7 @@ static int sock_ep_rx_ctx(struct fid_ep *ep, int index, struct fi_rx_attr *attr,
 	if (!rx_ctx)
 		return -FI_ENOMEM;
 
-	rx_ctx->rx_id = index;
+	rx_ctx->rx_id = (uint16_t) index;
 	rx_ctx->ep_attr = sock_ep->attr;
 	rx_ctx->domain = sock_ep->attr->domain;
 	rx_ctx->av = sock_ep->attr->av;
@@ -1163,8 +1197,8 @@ static int sock_ep_rx_ctx(struct fid_ep *ep, int index, struct fi_rx_attr *attr,
 	rx_ctx->min_multi_recv = sock_ep->attr->min_multi_recv;
 	*rx_ep = &rx_ctx->ctx;
 	sock_ep->attr->rx_array[index] = rx_ctx;
-	atomic_inc(&sock_ep->attr->num_rx_ctx);
-	atomic_inc(&sock_ep->attr->domain->ref);
+	ofi_atomic_inc32(&sock_ep->attr->num_rx_ctx);
+	ofi_atomic_inc32(&sock_ep->attr->domain->ref);
 	return 0;
 }
 
@@ -1215,9 +1249,12 @@ int sock_stx_ctx(struct fid_domain *domain,
 		return -FI_ENOMEM;
 
 	tx_ctx->domain = dom;
+	if (tx_ctx->rx_ctrl_ctx && tx_ctx->rx_ctrl_ctx->is_ctrl_ctx)
+		tx_ctx->rx_ctrl_ctx->domain = dom;
+
 	tx_ctx->fid.stx.fid.ops = &sock_ctx_ops;
 	tx_ctx->fid.stx.ops = &sock_ep_ops;
-	atomic_inc(&dom->ref);
+	ofi_atomic_inc32(&dom->ref);
 
 	*stx = &tx_ctx->fid.stx;
 	return 0;
@@ -1231,12 +1268,6 @@ static int sock_verify_rx_attr(const struct fi_rx_attr *attr)
 	if ((attr->msg_order | SOCK_EP_MSG_ORDER) != SOCK_EP_MSG_ORDER)
 		return -FI_ENODATA;
 
-	if ((attr->comp_order | SOCK_EP_COMP_ORDER) != SOCK_EP_COMP_ORDER)
-		return -FI_ENODATA;
-
-	if (attr->total_buffered_recv > SOCK_EP_MAX_BUFF_RECV)
-		return -FI_ENODATA;
-
 	if (sock_get_tx_size(attr->size) > sock_get_tx_size(SOCK_EP_TX_SZ))
 		return -FI_ENODATA;
 
@@ -1272,50 +1303,52 @@ int sock_srx_ctx(struct fid_domain *domain,
 	/* default config */
 	rx_ctx->min_multi_recv = SOCK_EP_MIN_MULTI_RECV;
 	*srx = &rx_ctx->ctx;
-	atomic_inc(&dom->ref);
+	ofi_atomic_inc32(&dom->ref);
 	return 0;
 }
 
-int sock_get_prefix_len(uint32_t net_addr)
-{
-	int count = 0;
-	while (net_addr > 0) {
-		net_addr = net_addr >> 1;
-		count++;
-	}
-	return count;
-}
-
 #if HAVE_GETIFADDRS
-char *sock_get_fabric_name(struct sockaddr_in *src_addr)
+static char *sock_get_fabric_name(struct sockaddr *src_addr)
 {
 	int ret;
         struct ifaddrs *ifaddrs, *ifa;
 	char *fabric_name = NULL;
-	struct in_addr net_in_addr;
-	struct sockaddr_in *host_addr, *net_addr;
-	char netbuf[SOCK_MAX_NETWORK_ADDR_SZ];
+	union ofi_sock_ip net_in_addr;
+	char netbuf[OFI_ADDRSTRLEN];
 	int prefix_len;
 
-	ret = getifaddrs(&ifaddrs);
+	ret = ofi_getifaddrs(&ifaddrs);
 	if (ret)
 		return NULL;
 
 	for (ifa = ifaddrs; ifa != NULL; ifa = ifa->ifa_next) {
-		if (ifa->ifa_addr == NULL || !(ifa->ifa_flags & IFF_UP) ||
-		     (ifa->ifa_addr->sa_family != AF_INET))
+		if (ifa->ifa_addr == NULL || !(ifa->ifa_flags & IFF_UP))
 			continue;
-		if (ofi_equals_ipaddr((struct sockaddr_in *)ifa->ifa_addr, src_addr)) {
-			host_addr = (struct sockaddr_in *)ifa->ifa_addr;
-			net_addr = (struct sockaddr_in *)ifa->ifa_netmask;
-			/* set fabric name to the network_adress in the format of a.b.c.d/e */
-			net_in_addr.s_addr = (uint32_t)((uint32_t) host_addr->sin_addr.s_addr &
-						(uint32_t) net_addr->sin_addr.s_addr);
-			inet_ntop(host_addr->sin_family, (void *)&(net_in_addr), netbuf,
-				   sizeof(netbuf));
-			prefix_len = sock_get_prefix_len(net_addr->sin_addr.s_addr);
+
+		if (ofi_equals_ipaddr(ifa->ifa_addr, src_addr)) {
+			prefix_len = (int) ofi_mask_addr(&net_in_addr.sa,
+							 ifa->ifa_addr,
+							 ifa->ifa_netmask);
+
+			switch (net_in_addr.sa.sa_family) {
+			case AF_INET:
+				inet_ntop(AF_INET,
+					&((struct sockaddr_in *)&net_in_addr)->sin_addr,
+					netbuf, sizeof(netbuf));
+				break;
+			case AF_INET6:
+				inet_ntop(AF_INET6,
+					&((struct sockaddr_in6 *)&net_in_addr)->sin6_addr,
+					netbuf, sizeof(netbuf));
+				break;
+			default:
+				snprintf(netbuf, sizeof(netbuf), "%s", "<unknown>");
+				netbuf[sizeof(netbuf)-1] = '\0';
+				break;
+			}
 			snprintf(netbuf + strlen(netbuf), sizeof(netbuf) - strlen(netbuf),
-				  "%s%d", "/", prefix_len);
+				 "%s%d", "/", prefix_len);
+			netbuf[sizeof(netbuf)-1] = '\0';
 			fabric_name = strdup(netbuf);
 			goto out;
 		}
@@ -1325,21 +1358,21 @@ out:
 	return fabric_name;
 }
 
-char *sock_get_domain_name(struct sockaddr_in *src_addr)
+char *sock_get_domain_name(struct sockaddr *src_addr)
 {
 	int ret;
         struct ifaddrs *ifaddrs, *ifa;
 	char *domain_name = NULL;
 
-	ret = getifaddrs(&ifaddrs);
+	ret = ofi_getifaddrs(&ifaddrs);
 	if (ret)
 		return NULL;
 
 	for (ifa = ifaddrs; ifa != NULL; ifa = ifa->ifa_next) {
-		if (ifa->ifa_addr == NULL || !(ifa->ifa_flags & IFF_UP) ||
-		     (ifa->ifa_addr->sa_family != AF_INET))
+		if (ifa->ifa_addr == NULL || !(ifa->ifa_flags & IFF_UP))
 			continue;
-		if (ofi_equals_ipaddr((struct sockaddr_in *)ifa->ifa_addr, src_addr)) {
+
+		if (ofi_equals_ipaddr(ifa->ifa_addr, src_addr)) {
 			domain_name = strdup(ifa->ifa_name);
 			goto out;
 		}
@@ -1349,12 +1382,12 @@ out:
 	return domain_name;
 }
 #else
-char *sock_get_fabric_name(struct sockaddr_in *src_addr)
+static char *sock_get_fabric_name(struct sockaddr *src_addr)
 {
 	return NULL;
 }
 
-char *sock_get_domain_name(struct sockaddr_in *src_addr)
+char *sock_get_domain_name(struct sockaddr *src_addr)
 {
 	return NULL;
 }
@@ -1378,10 +1411,14 @@ static void sock_set_fabric_attr(void *src_addr, const struct fi_fabric_attr *hi
 	if (!attr->name)
 		attr->name = strdup(sock_fab_name);
 
-	attr->prov_name = NULL;
+	if (hint_attr) {
+		attr->prov_name = strdup(hint_attr->prov_name);
+		attr->api_version = hint_attr->api_version;
+	}
 }
 
-static void sock_set_domain_attr(void *src_addr, const struct fi_domain_attr *hint_attr,
+static void sock_set_domain_attr(uint32_t api_version, void *src_addr,
+				 const struct fi_domain_attr *hint_attr,
 				 struct fi_domain_attr *attr)
 {
 	struct sock_domain *domain;
@@ -1390,6 +1427,9 @@ static void sock_set_domain_attr(void *src_addr, const struct fi_domain_attr *hi
 	attr->domain = domain ? &domain->dom_fid : NULL;
 	if (!hint_attr) {
 		*attr = sock_domain_attr;
+
+		if (FI_VERSION_LT(api_version, FI_VERSION(1, 5)))
+			attr->mr_mode = OFI_MR_SCALABLE;
 		goto out;
 	}
 
@@ -1408,8 +1448,14 @@ static void sock_set_domain_attr(void *src_addr, const struct fi_domain_attr *hi
 		attr->control_progress = sock_domain_attr.control_progress;
 	if (attr->data_progress == FI_PROGRESS_UNSPEC)
 		attr->data_progress = sock_domain_attr.data_progress;
-	if (attr->mr_mode == FI_MR_UNSPEC)
-		attr->mr_mode = sock_domain_attr.mr_mode;
+	if (FI_VERSION_LT(api_version, FI_VERSION(1, 5))) {
+		if (attr->mr_mode == OFI_MR_UNSPEC)
+			attr->mr_mode = OFI_MR_SCALABLE;
+	} else {
+		if ((attr->mr_mode != OFI_MR_BASIC) &&
+		    (attr->mr_mode != OFI_MR_SCALABLE))
+			attr->mr_mode = 0;
+	}
 
 	if (attr->cq_cnt == 0)
 		attr->cq_cnt = sock_domain_attr.cq_cnt;
@@ -1423,6 +1469,14 @@ static void sock_set_domain_attr(void *src_addr, const struct fi_domain_attr *hi
 		attr->max_ep_tx_ctx = sock_domain_attr.max_ep_tx_ctx;
 	if (attr->max_ep_rx_ctx == 0)
 		attr->max_ep_rx_ctx = sock_domain_attr.max_ep_rx_ctx;
+	if (attr->max_ep_stx_ctx == 0)
+		attr->max_ep_stx_ctx = sock_domain_attr.max_ep_stx_ctx;
+	if (attr->max_ep_srx_ctx == 0)
+		attr->max_ep_srx_ctx = sock_domain_attr.max_ep_srx_ctx;
+	if (attr->cntr_cnt == 0)
+		attr->cntr_cnt = sock_domain_attr.cntr_cnt;
+	if (attr->mr_iov_limit == 0)
+		attr->mr_iov_limit = sock_domain_attr.mr_iov_limit;
 
 	attr->mr_key_size = sock_domain_attr.mr_key_size;
 	attr->cq_data_size = sock_domain_attr.cq_data_size;
@@ -1435,101 +1489,93 @@ out:
 }
 
 
-struct fi_info *sock_fi_info(enum fi_ep_type ep_type, struct fi_info *hints,
-			     void *src_addr, void *dest_addr)
+struct fi_info *sock_fi_info(uint32_t version, enum fi_ep_type ep_type,
+			     const struct fi_info *hints, void *src_addr,
+			     void *dest_addr)
 {
 	struct fi_info *info;
 
-	info = fi_allocinfo();
+	info = fi_dupinfo(hints);
 	if (!info)
 		return NULL;
+	free(info->src_addr);
+	free(info->dest_addr);
+	info->src_addr = NULL;
+	info->dest_addr = NULL;
+	info->src_addrlen = 0;
+	info->dest_addrlen = 0;
 
-	info->src_addr = calloc(1, sizeof(struct sockaddr_in));
+	assert(src_addr);
+	info->src_addr = calloc(1, ofi_sizeofaddr(src_addr));
 	if (!info->src_addr)
 		goto err;
 
 	info->mode = SOCK_MODE;
-	info->addr_format = FI_SOCKADDR_IN;
-
-	if (src_addr)
-		memcpy(info->src_addr, src_addr, sizeof(struct sockaddr_in));
+	memcpy(info->src_addr, src_addr, ofi_sizeofaddr(src_addr));
+	info->src_addrlen = ofi_sizeofaddr(info->src_addr);
+	if (info->src_addrlen == sizeof(struct sockaddr_in6))
+		info->addr_format = FI_SOCKADDR_IN6;
 	else
-		sock_get_src_addr_from_hostname(info->src_addr, NULL);
-	info->src_addrlen = sizeof(struct sockaddr_in);
+		info->addr_format = FI_SOCKADDR_IN;
 
 	if (dest_addr) {
-		info->dest_addr = calloc(1, sizeof(struct sockaddr_in));
+		info->dest_addr = calloc(1, ofi_sizeofaddr(dest_addr));
 		if (!info->dest_addr)
 			goto err;
-		info->dest_addrlen = sizeof(struct sockaddr_in);
-		memcpy(info->dest_addr, dest_addr, sizeof(struct sockaddr_in));
+		info->dest_addrlen = ofi_sizeofaddr(dest_addr);
+		memcpy(info->dest_addr, dest_addr, info->dest_addrlen);
 	}
 
 	if (hints) {
-		if (hints->caps)
-			info->caps = hints->caps;
-
-		if (hints->ep_attr)
-			*(info->ep_attr) = *(hints->ep_attr);
-
-		if (hints->tx_attr)
-			*(info->tx_attr) = *(hints->tx_attr);
-
-		if (hints->rx_attr)
-			*(info->rx_attr) = *(hints->rx_attr);
-
-		if (hints->handle)
-			info->handle = hints->handle;
-
-		sock_set_domain_attr(info->src_addr, hints->domain_attr, info->domain_attr);
+		sock_set_domain_attr(version, info->src_addr, hints->domain_attr,
+				     info->domain_attr);
 		sock_set_fabric_attr(info->src_addr, hints->fabric_attr, info->fabric_attr);
 	} else {
-		sock_set_domain_attr(info->src_addr, NULL, info->domain_attr);
+		sock_set_domain_attr(version, info->src_addr, NULL,
+				     info->domain_attr);
 		sock_set_fabric_attr(info->src_addr, NULL, info->fabric_attr);
 	}
 
 	info->ep_attr->type = ep_type;
 	return info;
 err:
-	free(info->src_addr);
-	free(info->dest_addr);
-	free(info);
+	fi_freeinfo(info);
 	return NULL;
 }
 
-int sock_get_src_addr_from_hostname(struct sockaddr_in *src_addr,
-					const char *service)
+int sock_get_src_addr_from_hostname(union ofi_sock_ip *src_addr,
+				    const char *service, uint16_t sa_family)
 {
 	int ret;
 	struct addrinfo ai, *rai = NULL;
 	char hostname[HOST_NAME_MAX];
 
 	memset(&ai, 0, sizeof(ai));
-	ai.ai_family = AF_INET;
+	ai.ai_family = sa_family;
 	ai.ai_socktype = SOCK_STREAM;
 
-	ofi_getnodename(hostname, sizeof(hostname));
+	ofi_getnodename(sa_family, hostname, sizeof(hostname));
 	ret = getaddrinfo(hostname, service, &ai, &rai);
 	if (ret) {
 		SOCK_LOG_DBG("getaddrinfo failed!\n");
 		return -FI_EINVAL;
 	}
-	memcpy(src_addr, (struct sockaddr_in *)rai->ai_addr,
-		sizeof(*src_addr));
+	memcpy(src_addr, rai->ai_addr, rai->ai_addrlen);
 	freeaddrinfo(rai);
 	return 0;
 }
 
 static int sock_ep_assign_src_addr(struct sock_ep *sock_ep, struct fi_info *info)
 {
-	sock_ep->attr->src_addr = calloc(1, sizeof(struct sockaddr_in));
+	sock_ep->attr->src_addr = calloc(1, sizeof(union ofi_sock_ip));
 	if (!sock_ep->attr->src_addr)
 		return -FI_ENOMEM;
 
 	if (info && info->dest_addr)
 		return sock_get_src_addr(info->dest_addr, sock_ep->attr->src_addr);
 	else
-		return sock_get_src_addr_from_hostname(sock_ep->attr->src_addr, NULL);
+		return sock_get_src_addr_from_hostname(sock_ep->attr->src_addr,
+						       NULL, 0);
 }
 
 int sock_alloc_endpoint(struct fid_domain *domain, struct fi_info *info,
@@ -1541,16 +1587,10 @@ int sock_alloc_endpoint(struct fid_domain *domain, struct fi_info *info,
 	struct sock_rx_ctx *rx_ctx;
 	struct sock_domain *sock_dom;
 
-	if (info) {
-		ret = sock_verify_info(info);
-		if (ret) {
-			SOCK_LOG_DBG("Cannot support requested options!\n");
-			return -FI_EINVAL;
-		}
-	}
-
+	assert(info);
 	sock_dom = container_of(domain, struct sock_domain, dom_fid);
-	sock_ep = (struct sock_ep *) calloc(1, sizeof(*sock_ep));
+
+	sock_ep = calloc(1, sizeof(*sock_ep));
 	if (!sock_ep)
 		return -FI_ENOMEM;
 
@@ -1590,60 +1630,60 @@ int sock_alloc_endpoint(struct fid_domain *domain, struct fi_info *info,
 	sock_ep->attr->fclass = fclass;
 	*ep = sock_ep;
 
-	if (info) {
-		sock_ep->attr->info.caps = info->caps;
-		sock_ep->attr->info.addr_format = FI_SOCKADDR_IN;
+	sock_ep->attr->info.caps = info->caps;
+	sock_ep->attr->info.addr_format = info->addr_format;
 
-		if (info->ep_attr) {
-			sock_ep->attr->ep_type = info->ep_attr->type;
-			sock_ep->attr->ep_attr.tx_ctx_cnt = info->ep_attr->tx_ctx_cnt;
-			sock_ep->attr->ep_attr.rx_ctx_cnt = info->ep_attr->rx_ctx_cnt;
-		}
-
-		if (info->src_addr) {
-			sock_ep->attr->src_addr = calloc(1, sizeof(struct sockaddr_in));
-			if (!sock_ep->attr->src_addr) {
-				ret = -FI_ENOMEM;
-				goto err2;
-			}
-			memcpy(sock_ep->attr->src_addr, info->src_addr,
-			       sizeof(struct sockaddr_in));
-		}
-
-		if (info->dest_addr) {
-			sock_ep->attr->dest_addr = calloc(1, sizeof(struct sockaddr_in));
-			if (!sock_ep->attr->dest_addr) {
-				ret = -FI_ENOMEM;
-				goto err2;
-			}
-			memcpy(sock_ep->attr->dest_addr, info->dest_addr,
-			       sizeof(struct sockaddr_in));
-		}
-
-		if (info->tx_attr) {
-			sock_ep->tx_attr = *info->tx_attr;
-			if (!(sock_ep->tx_attr.op_flags & (FI_INJECT_COMPLETE |
-			     FI_TRANSMIT_COMPLETE | FI_DELIVERY_COMPLETE)))
-                        	sock_ep->tx_attr.op_flags |= FI_TRANSMIT_COMPLETE;
-			sock_ep->tx_attr.size = sock_ep->tx_attr.size ?
-				sock_ep->tx_attr.size : SOCK_EP_TX_SZ;
-		}
-
-		if (info->rx_attr)
-			sock_ep->rx_attr = *info->rx_attr;
-		sock_ep->attr->info.handle = info->handle;
+	if (info->ep_attr) {
+		sock_ep->attr->ep_type = info->ep_attr->type;
+		sock_ep->attr->ep_attr.tx_ctx_cnt = info->ep_attr->tx_ctx_cnt;
+		sock_ep->attr->ep_attr.rx_ctx_cnt = info->ep_attr->rx_ctx_cnt;
 	}
 
+	if (info->src_addr) {
+		sock_ep->attr->src_addr = calloc(1, sizeof(*sock_ep->
+							   attr->src_addr));
+		if (!sock_ep->attr->src_addr) {
+			ret = -FI_ENOMEM;
+			goto err2;
+		}
+		memcpy(sock_ep->attr->src_addr, info->src_addr,
+			info->src_addrlen);
+	}
+
+	if (info->dest_addr) {
+		sock_ep->attr->dest_addr = calloc(1, sizeof(*sock_ep->
+							    attr->dest_addr));
+		if (!sock_ep->attr->dest_addr) {
+			ret = -FI_ENOMEM;
+			goto err2;
+		}
+		memcpy(sock_ep->attr->dest_addr, info->dest_addr,
+			info->dest_addrlen);
+	}
+
+	if (info->tx_attr) {
+		sock_ep->tx_attr = *info->tx_attr;
+		if (!(sock_ep->tx_attr.op_flags & (FI_INJECT_COMPLETE |
+			FI_TRANSMIT_COMPLETE | FI_DELIVERY_COMPLETE)))
+			sock_ep->tx_attr.op_flags |= FI_TRANSMIT_COMPLETE;
+		sock_ep->tx_attr.size = sock_ep->tx_attr.size ?
+			sock_ep->tx_attr.size : SOCK_EP_TX_SZ;
+	}
+
+	if (info->rx_attr)
+		sock_ep->rx_attr = *info->rx_attr;
+	sock_ep->attr->info.handle = info->handle;
+
 	if (!sock_ep->attr->src_addr && sock_ep_assign_src_addr(sock_ep, info)) {
 		SOCK_LOG_ERROR("failed to get src_address\n");
 		ret = -FI_EINVAL;
 		goto err2;
 	}
 
-	atomic_initialize(&sock_ep->attr->ref, 0);
-	atomic_initialize(&sock_ep->attr->num_tx_ctx, 0);
-	atomic_initialize(&sock_ep->attr->num_rx_ctx, 0);
-	fastlock_init(&sock_ep->attr->lock);
+	ofi_atomic_initialize32(&sock_ep->attr->ref, 0);
+	ofi_atomic_initialize32(&sock_ep->attr->num_tx_ctx, 0);
+	ofi_atomic_initialize32(&sock_ep->attr->num_rx_ctx, 0);
+	ofi_mutex_init(&sock_ep->attr->lock);
 
 	if (sock_ep->attr->ep_attr.tx_ctx_cnt == FI_SHARED_CONTEXT)
 		sock_ep->attr->tx_shared = 1;
@@ -1679,6 +1719,8 @@ int sock_alloc_endpoint(struct fid_domain *domain, struct fi_info *info,
 		}
 		tx_ctx->ep_attr = sock_ep->attr;
 		tx_ctx->domain = sock_dom;
+		if (tx_ctx->rx_ctrl_ctx && tx_ctx->rx_ctrl_ctx->is_ctrl_ctx)
+			tx_ctx->rx_ctrl_ctx->domain = sock_dom;
 		tx_ctx->tx_id = 0;
 		dlist_insert_tail(&sock_ep->attr->tx_ctx_entry, &tx_ctx->ep_list);
 		sock_ep->attr->tx_array[0] = tx_ctx;
@@ -1702,30 +1744,18 @@ int sock_alloc_endpoint(struct fid_domain *domain, struct fi_info *info,
 	/* default config */
 	sock_ep->attr->min_multi_recv = SOCK_EP_MIN_MULTI_RECV;
 
-	if (info)
-		memcpy(&sock_ep->attr->info, info, sizeof(struct fi_info));
+	memcpy(&sock_ep->attr->info, info, sizeof(struct fi_info));
 
 	sock_ep->attr->domain = sock_dom;
-	fastlock_init(&sock_ep->attr->cm.lock);
-	if (sock_ep->attr->ep_type == FI_EP_MSG) {
-		dlist_init(&sock_ep->attr->cm.msg_list);
-		if (socketpair(AF_UNIX, SOCK_STREAM, 0,
-			       sock_ep->attr->cm.signal_fds) < 0) {
-			ret = -FI_EINVAL;
-			goto err2;
-		}
-
-		if (fi_fd_nonblock(sock_ep->attr->cm.signal_fds[1]))
-			SOCK_LOG_ERROR("fi_fd_nonblock failed");
-	}
+	ofi_mutex_init(&sock_ep->attr->cm.lock);
 
 	if (sock_conn_map_init(sock_ep, sock_cm_def_map_sz)) {
-		SOCK_LOG_ERROR("failed to init connection map: %s\n", strerror(errno));
+		SOCK_LOG_ERROR("failed to init connection map\n");
 		ret = -FI_EINVAL;
 		goto err2;
 	}
 
-	atomic_inc(&sock_dom->ref);
+	ofi_atomic_inc32(&sock_dom->ref);
 	return 0;
 
 err2:
@@ -1741,30 +1771,63 @@ err1:
 
 void sock_ep_remove_conn(struct sock_ep_attr *attr, struct sock_conn *conn)
 {
+	if (attr->cmap.used <= 0 || conn->sock_fd == -1)
+		return;
 	sock_pe_poll_del(attr->domain->pe, conn->sock_fd);
-	idm_clear(&attr->conn_idm, conn->sock_fd);
 	sock_conn_release_entry(&attr->cmap, conn);
 }
 
 struct sock_conn *sock_ep_lookup_conn(struct sock_ep_attr *attr, fi_addr_t index,
-					struct sockaddr_in *addr)
+				      union ofi_sock_ip *addr)
 {
 	int i;
-	uint16_t idx;
+	uint64_t idx;
+	char buf[8];
 	struct sock_conn *conn;
 
 	idx = (attr->ep_type == FI_EP_MSG) ? index : index & attr->av->mask;
 
-	conn = idm_lookup(&attr->av_idm, idx);
-	if (conn && conn != SOCK_CM_CONN_IN_PROGRESS)
+	conn = ofi_idm_lookup(&attr->av_idm, (int) idx);
+	if (conn && conn != SOCK_CM_CONN_IN_PROGRESS) {
+		/* Verify that the existing connection is still usable, and
+		 * that the peer didn't restart.
+		 */
+		if (conn->connected == 0 ||
+		    (sock_comm_peek(conn, buf, 8) == 0 && conn->connected == 0)) {
+			sock_ep_remove_conn(attr, conn);
+			ofi_straddr_log(&sock_prov, FI_LOG_WARN, FI_LOG_EP_CTRL,
+					"Peer disconnected", &addr->sa);
+			return NULL;
+		}
+		if (conn->av_index != FI_ADDR_NOTAVAIL)
+			assert(conn->av_index == idx);
+		else
+			conn->av_index = idx;
 		return conn;
+	}
 
 	for (i = 0; i < attr->cmap.used; i++) {
 		if (!attr->cmap.table[i].connected)
 			continue;
 
-		if (ofi_equals_sockaddr(&attr->cmap.table[i].addr, addr))
-			return &attr->cmap.table[i];
+		if (ofi_equals_sockaddr(&attr->cmap.table[i].addr.sa, &addr->sa) &&
+			attr->cmap.table[i].av_index == idx) {
+			conn = &attr->cmap.table[i];
+			break;
+		}
+	}
+	if (conn && conn != SOCK_CM_CONN_IN_PROGRESS) {
+		if (conn->connected == 0 ||
+		    (sock_comm_peek(conn, buf, 8) == 0 && conn->connected == 0)) {
+			sock_ep_remove_conn(attr, conn);
+			ofi_straddr_log(&sock_prov, FI_LOG_WARN, FI_LOG_EP_CTRL,
+					"Peer disconnected", &addr->sa);
+			return NULL;
+		}
+		if (conn->av_index != FI_ADDR_NOTAVAIL)
+			assert(conn->av_index == idx);
+		else
+			conn->av_index = idx;
 	}
 	return conn;
 }
@@ -1773,34 +1836,41 @@ int sock_ep_get_conn(struct sock_ep_attr *attr, struct sock_tx_ctx *tx_ctx,
 		     fi_addr_t index, struct sock_conn **pconn)
 {
 	struct sock_conn *conn;
-	uint64_t av_index = (attr->ep_type == FI_EP_MSG) ? 0 : (index & attr->av->mask);
-	struct sockaddr_in *addr;
+	uint64_t av_index = (attr->ep_type == FI_EP_MSG) ?
+			    0 : (index & attr->av->mask);
+	union ofi_sock_ip *addr;
+	int ret = FI_SUCCESS;
 
 	if (attr->ep_type == FI_EP_MSG)
 		addr = attr->dest_addr;
-	else
-		addr = (struct sockaddr_in *)&attr->av->table[av_index].addr;
+	else {
+		ofi_mutex_lock(&attr->av->table_lock);
+		addr = &attr->av->table[av_index].addr;
+		ofi_mutex_unlock(&attr->av->table_lock);
+	}
 
-	fastlock_acquire(&attr->cmap.lock);
+	ofi_mutex_lock(&attr->cmap.lock);
 	conn = sock_ep_lookup_conn(attr, av_index, addr);
 	if (!conn) {
 		conn = SOCK_CM_CONN_IN_PROGRESS;
-		if (idm_set(&attr->av_idm, av_index, conn) < 0)
-			SOCK_LOG_ERROR("idm_set failed\n");
+		if (ofi_idm_set(&attr->av_idm, (int) av_index, conn) < 0)
+			SOCK_LOG_ERROR("ofi_idm_set failed\n");
 	}
-	fastlock_release(&attr->cmap.lock);
+	ofi_mutex_unlock(&attr->cmap.lock);
 
 	if (conn == SOCK_CM_CONN_IN_PROGRESS)
-		conn = sock_ep_connect(attr, av_index);
+		ret = sock_ep_connect(attr, av_index, &conn);
 
 	if (!conn) {
-		SOCK_LOG_ERROR("Error in connecting: %s\n", strerror(errno));
-		if (errno == EINPROGRESS)
-			return -FI_EAGAIN;
-		else
-			return -errno;
+		SOCK_LOG_ERROR("Unable to find connection entry. "
+			       "Error in connecting: %s\n",
+			       fi_strerror(-ret));
+		ofi_straddr_log(&sock_prov, FI_LOG_WARN, FI_LOG_EP_CTRL,
+				"Unable to connect to", &addr->sa);
+		return -FI_ENOENT;
 	}
 
 	*pconn = conn;
-	return conn->address_published ? 0 : sock_conn_send_src_addr(attr, tx_ctx, conn);
+	return conn->address_published ?
+	       0 : (int) sock_conn_send_src_addr(attr, tx_ctx, conn);
 }
